from .colourpickers import *


class DefaultConfig:
    USE_DEFAULT_FILTERS = False

    COLOUR_PICKER_CLS = LayerScoreColourPicker

    RANKED_STYLES = {
        "node": {
            "fontname": "helvetica",
        },
        "edge": {

        },
    }



from os import getcwd


class Errors:
    class ExternalFileError(FileNotFoundError):
        pass

    class NoValidParserError(NotImplementedError):
        pass


class Constants:
    __cwd = getcwd()
    if ("\\" in __cwd) and ("/" in __cwd):
        raise NotImplementedError("unable to work with path which contains both forward and back slashes")
    elif ("\\" not in __cwd) and ("/" not in __cwd):
        raise NotImplementedError("unable to detect whether paths will contain forward or back slashes")
    else:
        PATH_DELIMITER = "\\" if "\\" in __cwd else "/"
        NON_PATH_DELIMITER = "/" if "\\" in __cwd else "\\"



from re import compile

from ..constants import Constants as MainConstants


class Patterns:
    REQUIRE = {
        compile(r"require[ \t]*\(\s*(\"|\')((.|\s)*?)\1\s*\)"): 1  # `require('module')`
    }


class Constants:
    NODE_MODULES_FRAGMENT = MainConstants.PATH_DELIMITER + "node_modules" + MainConstants.PATH_DELIMITER



from abc import ABC
from os import path as ospath
from typing import Tuple, Callable, Generator, FrozenSet, Optional

from ..constants import Constants


class Parser(ABC):
    @property
    def FILTERS(self) -> FrozenSet[Callable[[str], bool]]:  # Abstract class constant
        raise NotImplementedError

    @property
    def NODE_ENDINGS(self) -> Tuple[str]:  # Abstract class constant
        raise NotImplementedError

    @classmethod
    def can_parse(cls, file_path: str) -> bool:
        """
        Indicates whether the file is supported by the parser class this is called on.
        For any files that should never be considered local dependencies, this should return False
        (for example, files inside a node_modules folder for JS projects)
        """

        raise NotImplementedError

    @classmethod
    def parse(cls, file_contents: str, file_dir: str, working_dir: str) -> Generator[str, None, None]:
        raise NotImplementedError

    @classmethod
    def _get_import_target(cls, import_node_starting_dir: str, import_node: str) -> Optional[str]:
        working_target = import_node_starting_dir + Constants.PATH_DELIMITER
        working_target += import_node.replace(".", Constants.PATH_DELIMITER).replace(Constants.NON_PATH_DELIMITER, Constants.PATH_DELIMITER)

        for path_ending in cls.NODE_ENDINGS:
            result = working_target + path_ending
            if ospath.isfile(result):
                return result



from typing import Generator
from re import compile
from os import path as ospath

from .regexparser import RegexParser
from .constants import Patterns, Constants as ParserConstants
from ..constants import Constants


class JsParser(RegexParser):
    FILTERS = frozenset((
        # TODO
    ))

    NODE_ENDINGS = (
        ".js",
        ".jsx",
        ".ts",
        ".tsx",
        ""
    )  # TODO: Possibly missing entries, and need to determine correct ordering to match JS's

    PATTERNS = {
        **Patterns.REQUIRE,
        compile(r"import [\w\{\}\s,]*? from (\"|\')((.|\s)*?)\1"): 1,  # `import _ from 'module'`
        compile(r"import (\"|\')((.|\s)*?)\1"): 1,  # `import 'module'`
    }  # TODO: Add all styles of import

    @classmethod
    def can_parse(cls, file_path: str) -> bool:
        result = file_path[-3:] in (".js", ".ts") or file_path[-4:] in (".jsx", ".tsx")
        result = result and (ParserConstants.NODE_MODULES_FRAGMENT not in file_path)
        return result

    @classmethod
    def parse(cls, file_contents: str, file_dir: str, working_dir: str) -> Generator[str, None, None]:
        for import_node in cls._get_import_nodes(file_contents, file_dir, working_dir):
            offset_file_dir = file_dir

            directory_offset_total = 0
            relative_chars = ""
            previous_char = None
            for char in import_node:
                if char == ".":
                    if previous_char == ".":
                        directory_offset_total += 1

                    relative_chars += char
                elif char in (Constants.PATH_DELIMITER, Constants.NON_PATH_DELIMITER):
                    relative_chars += char
                else:
                    break

                previous_char = char

            for i in range(directory_offset_total):
                offset_file_dir = ospath.dirname(offset_file_dir)  # Go up 1 directory

            import_node = import_node.lstrip(relative_chars)  # Remove relative characters from front of import node

            target_path = cls._get_import_target(offset_file_dir, import_node)

            if target_path:
                yield target_path
